@@ -8,28 +8,17 @@
  * Copyright (c) 2009-2013, Even Rouault <even dot rouault at spatialys.com>
  * Copyright (c) 2009, Frank Warmerdam
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_string.h"
 #include "gdal_pam.h"
+#include "gdal_colortable.h"
 #include "gdal_frmts.h"
+#include "gdal_driver.h"
+#include "gdal_drivermanager.h"
+#include "gdal_openinfo.h"
+#include "gdal_cpp_functions.h"
 #include "iso8211.h"
 #include "ogr_spatialref.h"
 
@@ -85,7 +74,7 @@ class SRPDataset final : public GDALPamDataset
     ~SRPDataset() override;
 
     const OGRSpatialReference *GetSpatialRef() const override;
-    CPLErr GetGeoTransform(double *padfGeoTransform) override;
+    CPLErr GetGeoTransform(GDALGeoTransform &gt) const override;
 
     char **GetMetadata(const char *pszDomain = "") override;
 
@@ -154,7 +143,7 @@ double SRPRasterBand::GetNoDataValue(int *pbSuccess)
 GDALColorInterp SRPRasterBand::GetColorInterpretation()
 
 {
-    SRPDataset *l_poDS = (SRPDataset *)this->poDS;
+    SRPDataset *l_poDS = cpl::down_cast<SRPDataset *>(poDS);
 
     if (l_poDS->oCT.GetColorEntryCount() > 0)
         return GCI_PaletteIndex;
@@ -169,7 +158,7 @@ GDALColorInterp SRPRasterBand::GetColorInterpretation()
 GDALColorTable *SRPRasterBand::GetColorTable()
 
 {
-    SRPDataset *l_poDS = (SRPDataset *)this->poDS;
+    SRPDataset *l_poDS = cpl::down_cast<SRPDataset *>(poDS);
 
     if (l_poDS->oCT.GetColorEntryCount() > 0)
         return &(l_poDS->oCT);
@@ -184,7 +173,7 @@ GDALColorTable *SRPRasterBand::GetColorTable()
 CPLErr SRPRasterBand::IReadBlock(int nBlockXOff, int nBlockYOff, void *pImage)
 
 {
-    SRPDataset *l_poDS = (SRPDataset *)this->poDS;
+    SRPDataset *l_poDS = cpl::down_cast<SRPDataset *>(poDS);
     vsi_l_offset offset;
     int nBlock = nBlockYOff * l_poDS->NFC + nBlockXOff;
     if (nBlockXOff >= l_poDS->NFC || nBlockYOff >= l_poDS->NFL)
@@ -392,7 +381,7 @@ const OGRSpatialReference *SRPDataset::GetSpatialRef() const
 /*                        GetGeoTransform()                             */
 /************************************************************************/
 
-CPLErr SRPDataset::GetGeoTransform(double *padfGeoTransform)
+CPLErr SRPDataset::GetGeoTransform(GDALGeoTransform &gt) const
 {
     if (EQUAL(osProduct, "ASRP"))
     {
@@ -401,49 +390,49 @@ CPLErr SRPDataset::GetGeoTransform(double *padfGeoTransform)
         if (ZNA == 9)
         {
             // North Polar Case
-            padfGeoTransform[0] = 111319.4907933 * (90.0 - PSO / 3600.0) *
-                                  sin(LSO * M_PI / 648000.0);
-            padfGeoTransform[1] = 40075016.68558 / ARV;
-            padfGeoTransform[2] = 0.0;
-            padfGeoTransform[3] = -111319.4907933 * (90.0 - PSO / 3600.0) *
-                                  cos(LSO * M_PI / 648000.0);
-            padfGeoTransform[4] = 0.0;
-            padfGeoTransform[5] = -40075016.68558 / ARV;
+            gt[0] = 111319.4907933 * (90.0 - PSO / 3600.0) *
+                    sin(LSO * M_PI / 648000.0);
+            gt[1] = 40075016.68558 / ARV;
+            gt[2] = 0.0;
+            gt[3] = -111319.4907933 * (90.0 - PSO / 3600.0) *
+                    cos(LSO * M_PI / 648000.0);
+            gt[4] = 0.0;
+            gt[5] = -40075016.68558 / ARV;
         }
         else if (ZNA == 18)
         {
             // South Polar Case
-            padfGeoTransform[0] = 111319.4907933 * (90.0 + PSO / 3600.0) *
-                                  sin(LSO * M_PI / 648000.0);
-            padfGeoTransform[1] = 40075016.68558 / ARV;
-            padfGeoTransform[2] = 0.0;
-            padfGeoTransform[3] = 111319.4907933 * (90.0 + PSO / 3600.0) *
-                                  cos(LSO * M_PI / 648000.0);
-            padfGeoTransform[4] = 0.0;
-            padfGeoTransform[5] = -40075016.68558 / ARV;
+            gt[0] = 111319.4907933 * (90.0 + PSO / 3600.0) *
+                    sin(LSO * M_PI / 648000.0);
+            gt[1] = 40075016.68558 / ARV;
+            gt[2] = 0.0;
+            gt[3] = 111319.4907933 * (90.0 + PSO / 3600.0) *
+                    cos(LSO * M_PI / 648000.0);
+            gt[4] = 0.0;
+            gt[5] = -40075016.68558 / ARV;
         }
         else
         {
             if (BRV == 0)
                 return CE_Failure;
-            padfGeoTransform[0] = LSO / 3600.0;
-            padfGeoTransform[1] = 360. / ARV;
-            padfGeoTransform[2] = 0.0;
-            padfGeoTransform[3] = PSO / 3600.0;
-            padfGeoTransform[4] = 0.0;
-            padfGeoTransform[5] = -360. / BRV;
+            gt[0] = LSO / 3600.0;
+            gt[1] = 360. / ARV;
+            gt[2] = 0.0;
+            gt[3] = PSO / 3600.0;
+            gt[4] = 0.0;
+            gt[5] = -360. / BRV;
         }
 
         return CE_None;
     }
     else if (EQUAL(osProduct, "USRP"))
     {
-        padfGeoTransform[0] = LSO;
-        padfGeoTransform[1] = LOD;
-        padfGeoTransform[2] = 0.0;
-        padfGeoTransform[3] = PSO;
-        padfGeoTransform[4] = 0.0;
-        padfGeoTransform[5] = -LAD;
+        gt[0] = LSO;
+        gt[1] = LOD;
+        gt[2] = 0.0;
+        gt[3] = PSO;
+        gt[4] = 0.0;
+        gt[5] = -LAD;
         return CE_None;
     }
 
@@ -532,13 +521,17 @@ bool SRPDataset::GetFromRecord(const char *pszFileName, DDFRecord *record)
         record->GetStringSubfield("SPR", 0, "BAD", 0, &bSuccess);
     if (pszBAD == nullptr)
         return false;
-    const CPLString osBAD = pszBAD;
-    {
-        char *c = (char *)strchr(osBAD, ' ');
-        if (c)
-            *c = 0;
-    }
+    std::string osBAD = pszBAD;
+    const auto nSpacePos = osBAD.find(' ');
+    if (nSpacePos != std::string::npos)
+        osBAD.resize(nSpacePos);
     CPLDebug("SRP", "BAD=%s", osBAD.c_str());
+    if (CPLHasPathTraversal(osBAD.c_str()))
+    {
+        CPLError(CE_Failure, CPLE_AppDefined, "Path traversal detected in %s",
+                 osBAD.c_str());
+        return false;
+    }
 
     /* -------------------------------------------------------------------- */
     /*      Read the tile map if available.                                 */
@@ -553,8 +546,9 @@ bool SRPDataset::GetFromRecord(const char *pszFileName, DDFRecord *record)
         if (field == nullptr)
             return false;
 
-        DDFFieldDefn *fieldDefn = field->GetFieldDefn();
-        DDFSubfieldDefn *subfieldDefn = fieldDefn->FindSubfieldDefn("TSI");
+        const DDFFieldDefn *fieldDefn = field->GetFieldDefn();
+        const DDFSubfieldDefn *subfieldDefn =
+            fieldDefn->FindSubfieldDefn("TSI");
         if (subfieldDefn == nullptr)
             return false;
 
@@ -595,8 +589,9 @@ bool SRPDataset::GetFromRecord(const char *pszFileName, DDFRecord *record)
     /*      Open the .IMG file.  Try to recover gracefully if the case      */
     /*      of the filename is wrong.                                       */
     /* -------------------------------------------------------------------- */
-    const CPLString osDirname = CPLGetDirname(pszFileName);
-    const CPLString osImgName = CPLFormCIFilename(osDirname, osBAD, nullptr);
+    const CPLString osDirname = CPLGetDirnameSafe(pszFileName);
+    const CPLString osImgName =
+        CPLFormCIFilenameSafe(osDirname, osBAD.c_str(), nullptr);
 
     fdIMG = VSIFOpenL(osImgName, "rb");
     if (fdIMG == nullptr)
@@ -686,8 +681,8 @@ bool SRPDataset::GetFromRecord(const char *pszFileName, DDFRecord *record)
     /* -------------------------------------------------------------------- */
     /*      Try to collect a color map from the .QAL file.                  */
     /* -------------------------------------------------------------------- */
-    const CPLString osBasename = CPLGetBasename(pszFileName);
-    osQALFileName = CPLFormCIFilename(osDirname, osBasename, "QAL");
+    const CPLString osBasename = CPLGetBasenameSafe(pszFileName);
+    osQALFileName = CPLFormCIFilenameSafe(osDirname, osBasename, "QAL");
 
     DDFModule oQALModule;
 
@@ -824,7 +819,7 @@ bool SRPDataset::GetFromRecord(const char *pszFileName, DDFRecord *record)
     }
     else
     {
-        if (std::abs(ZNA) >= 1 && std::abs(ZNA) <= 60)
+        if (ZNA >= -60 && ZNA <= 60 && ZNA != 0)
         {
             m_oSRS.SetUTM(std::abs(ZNA), ZNA > 0);
             m_oSRS.SetWellKnownGeogCS("WGS84");
@@ -1068,13 +1063,12 @@ char **SRPDataset::GetGENListFromTHF(const char *pszFileName)
     DDFRecord *record = nullptr;
     DDFField *field = nullptr;
     DDFFieldDefn *fieldDefn = nullptr;
-    int nFilenames = 0;
+    CPLStringList aosFilenames;
 
-    char **papszFileNames = nullptr;
     if (!module.Open(pszFileName, TRUE))
-        return papszFileNames;
+        return nullptr;
 
-    CPLString osDirName(CPLGetDirname(pszFileName));
+    const CPLString osDirName(CPLGetDirnameSafe(pszFileName));
 
     while (true)
     {
@@ -1135,7 +1129,14 @@ char **SRPDataset::GetGENListFromTHF(const char *pszFileName)
                      * characters */
                     CPLString osDirDataset = pszNAM;
                     osDirDataset.resize(6);
-                    CPLString osDatasetDir = CPLFormFilename(
+                    if (CPLHasPathTraversal(osDirDataset.c_str()))
+                    {
+                        CPLError(CE_Failure, CPLE_AppDefined,
+                                 "Path traversal detected in %s",
+                                 osDirDataset.c_str());
+                        return nullptr;
+                    }
+                    CPLString osDatasetDir = CPLFormFilenameSafe(
                         osDirName.c_str(), osDirDataset.c_str(), nullptr);
 
                     CPLString osGENFileName = "";
@@ -1150,10 +1151,11 @@ char **SRPDataset::GetGENListFromTHF(const char *pszFileName)
                         {
                             while (*ptrDir)
                             {
-                                if (EQUAL(CPLGetExtension(*ptrDir), "GEN"))
+                                if (EQUAL(CPLGetExtensionSafe(*ptrDir).c_str(),
+                                          "GEN"))
                                 {
                                     bFound = 1;
-                                    osGENFileName = CPLFormFilename(
+                                    osGENFileName = CPLFormFilenameSafe(
                                         osDatasetDir.c_str(), *ptrDir, nullptr);
                                     CPLDebug("SRP",
                                              "Building GEN full file name : %s",
@@ -1176,11 +1178,13 @@ char **SRPDataset::GetGENListFromTHF(const char *pszFileName)
                         {
                             while (*ptrDir)
                             {
-                                if (EQUAL(CPLGetExtension(*ptrDir), "GEN") &&
-                                    EQUALN(CPLGetBasename(*ptrDir), osName, 6))
+                                if (EQUAL(CPLGetExtensionSafe(*ptrDir).c_str(),
+                                          "GEN") &&
+                                    EQUALN(CPLGetBasenameSafe(*ptrDir).c_str(),
+                                           osName, 6))
                                 {
                                     bFound = 1;
-                                    osGENFileName = CPLFormFilename(
+                                    osGENFileName = CPLFormFilenameSafe(
                                         osDirName.c_str(), *ptrDir, nullptr);
                                     CPLDebug("SRP",
                                              "Building GEN full file name : %s",
@@ -1195,18 +1199,13 @@ char **SRPDataset::GetGENListFromTHF(const char *pszFileName)
 
                     if (bFound == 1)
                     {
-                        papszFileNames = (char **)CPLRealloc(
-                            papszFileNames, sizeof(char *) * (nFilenames + 2));
-                        papszFileNames[nFilenames] =
-                            CPLStrdup(osGENFileName.c_str());
-                        papszFileNames[nFilenames + 1] = nullptr;
-                        nFilenames++;
+                        aosFilenames.AddString(osGENFileName.c_str());
                     }
                 }
             }
         }
     }
-    return papszFileNames;
+    return aosFilenames.StealList();
 }
 
 /************************************************************************/
@@ -1406,23 +1405,27 @@ char **SRPDataset::GetIMGListFromGEN(const char *pszFileName,
             const char *pszBAD = record->GetStringSubfield("SPR", 0, "BAD", 0);
             if (pszBAD == nullptr || strlen(pszBAD) != 12)
                 continue;
-            CPLString osBAD = pszBAD;
-            {
-                char *c = (char *)strchr(osBAD.c_str(), ' ');
-                if (c)
-                    *c = 0;
-            }
+            std::string osBAD = pszBAD;
+            const auto nSpacePos = osBAD.find(' ');
+            if (nSpacePos != std::string::npos)
+                osBAD.resize(nSpacePos);
             CPLDebug("SRP", "BAD=%s", osBAD.c_str());
+            if (CPLHasPathTraversal(osBAD.c_str()))
+            {
+                CPLError(CE_Failure, CPLE_AppDefined,
+                         "Path traversal detected in %s", osBAD.c_str());
+                return nullptr;
+            }
 
             /* Build full IMG file name from BAD value */
-            const CPLString osGENDir(CPLGetDirname(pszFileName));
+            const CPLString osGENDir(CPLGetDirnameSafe(pszFileName));
 
-            const CPLString osFileName =
-                CPLFormFilename(osGENDir.c_str(), osBAD.c_str(), nullptr);
+            std::string osFileName =
+                CPLFormFilenameSafe(osGENDir.c_str(), osBAD.c_str(), nullptr);
             VSIStatBufL sStatBuf;
-            if (VSIStatL(osFileName, &sStatBuf) == 0)
+            if (VSIStatL(osFileName.c_str(), &sStatBuf) == 0)
             {
-                osBAD = osFileName;
+                osBAD = std::move(osFileName);
                 CPLDebug("SRP", "Building IMG full file name : %s",
                          osBAD.c_str());
             }
@@ -1441,8 +1444,8 @@ char **SRPDataset::GetIMGListFromGEN(const char *pszFileName,
                 {
                     if (EQUAL(*ptrDir, osBAD.c_str()))
                     {
-                        osBAD =
-                            CPLFormFilename(osGENDir.c_str(), *ptrDir, nullptr);
+                        osBAD = CPLFormFilenameSafe(osGENDir.c_str(), *ptrDir,
+                                                    nullptr);
                         CPLDebug("SRP", "Building IMG full file name : %s",
                                  osBAD.c_str());
                         break;
@@ -1496,7 +1499,7 @@ GDALDataset *SRPDataset::Open(GDALOpenInfo *poOpenInfo)
             return nullptr;
         CPLString osFileName(poOpenInfo->pszFilename);
 
-        if (EQUAL(CPLGetExtension(osFileName.c_str()), "THF"))
+        if (EQUAL(CPLGetExtensionSafe(osFileName.c_str()).c_str(), "THF"))
         {
 
             CPLDebug("SRP", "Read THF");
@@ -1537,7 +1540,7 @@ GDALDataset *SRPDataset::Open(GDALOpenInfo *poOpenInfo)
 
         if (bTHFWithSingleGEN
 #ifdef OPEN_GEN
-            || EQUAL(CPLGetExtension(osFileName.c_str()), "GEN")
+            || EQUAL(CPLGetExtensionSafe(osFileName.c_str()).c_str(), "GEN")
 #endif
         )
         {
@@ -1566,7 +1569,7 @@ GDALDataset *SRPDataset::Open(GDALOpenInfo *poOpenInfo)
             }
         }
 
-        if (EQUAL(CPLGetExtension(osFileName.c_str()), "IMG"))
+        if (EQUAL(CPLGetExtensionSafe(osFileName.c_str()).c_str(), "IMG"))
         {
 
             osIMGFileName = osFileName;
@@ -1597,7 +1600,7 @@ GDALDataset *SRPDataset::Open(GDALOpenInfo *poOpenInfo)
             // --------------------------------------------------------------------
             VSIStatBufL sStatBuf;
 
-            CPLString basename = CPLGetBasename(osFileName);
+            CPLString basename = CPLGetBasenameSafe(osFileName);
             if (basename.size() != 8)
             {
                 CPLDebug("SRP", "Invalid basename file");
@@ -1606,14 +1609,14 @@ GDALDataset *SRPDataset::Open(GDALOpenInfo *poOpenInfo)
 
             nRecordIndex = static_cast<int>(CPLScanLong(basename + 6, 2));
 
-            CPLString path = CPLGetDirname(osFileName);
+            CPLString path = CPLGetDirnameSafe(osFileName);
             CPLString basename01 = ResetTo01(basename);
-            osFileName = CPLFormFilename(path, basename01, ".IMG");
+            osFileName = CPLFormFilenameSafe(path, basename01, ".IMG");
 
-            osFileName = CPLResetExtension(osFileName, "GEN");
+            osFileName = CPLResetExtensionSafe(osFileName, "GEN");
             if (VSIStatL(osFileName, &sStatBuf) != 0)
             {
-                osFileName = CPLResetExtension(osFileName, "gen");
+                osFileName = CPLResetExtensionSafe(osFileName, "gen");
                 if (VSIStatL(osFileName, &sStatBuf) != 0)
                     return nullptr;
             }
@@ -1627,9 +1630,7 @@ GDALDataset *SRPDataset::Open(GDALOpenInfo *poOpenInfo)
 
         if (poOpenInfo->eAccess == GA_Update)
         {
-            CPLError(CE_Failure, CPLE_NotSupported,
-                     "The SRP driver does not support update access to existing"
-                     " datasets.\n");
+            ReportUpdateNotSupportedByDriver("SRP");
             return nullptr;
         }
 
