@@ -8,25 +8,7 @@
  * Copyright (c) 2009-2010, 2012-2015, Martin Landa <landa.martin gmail.com>
  * Copyright (c) 2012, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person
- * obtaining a copy of this software and associated documentation
- * files (the "Software"), to deal in the Software without
- * restriction, including without limitation the rights to use, copy,
- * modify, merge, publish, distribute, sublicense, and/or sell copies
- * of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be
- * included in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
- * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
- * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
- * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "vfkreader.h"
@@ -785,8 +767,8 @@ bool VFKFeature::LoadGeometryPoint()
 */
 bool VFKFeature::LoadGeometryLineStringSBP()
 {
-    VFKDataBlock *poDataBlockPoints =
-        (VFKDataBlock *)m_poDataBlock->GetReader()->GetDataBlock("SOBR");
+    VFKDataBlock *poDataBlockPoints = cpl::down_cast<VFKDataBlock *>(
+        m_poDataBlock->GetReader()->GetDataBlock("SOBR"));
     if (!poDataBlockPoints)
         return false;
 
@@ -823,7 +805,7 @@ bool VFKFeature::LoadGeometryLineStringSBP()
         const OGRPoint *pt = poPoint->GetGeometry()->toPoint();
         OGRLine.addPoint(pt);
 
-        poLine = (VFKFeature *)m_poDataBlock->GetNextFeature();
+        poLine = cpl::down_cast<VFKFeature *>(m_poDataBlock->GetNextFeature());
         if (!poLine)
             break;
     };
@@ -846,8 +828,8 @@ bool VFKFeature::LoadGeometryLineStringSBP()
 */
 bool VFKFeature::LoadGeometryLineStringHP()
 {
-    VFKDataBlock *poDataBlockLines =
-        (VFKDataBlock *)m_poDataBlock->GetReader()->GetDataBlock("SBP");
+    VFKDataBlock *poDataBlockLines = cpl::down_cast<VFKDataBlock *>(
+        m_poDataBlock->GetReader()->GetDataBlock("SBP"));
     if (!poDataBlockLines)
         return false;
 
